# -*- coding: utf-8 -*-

# sample is mpaf13_9_19

# import utils_fit
import fit_T_dep
import matplotlib.pyplot as plt
import numpy as np
import scipy.constants

KB = scipy.constants.Boltzmann
hbar = scipy.constants.hbar

# --- plotting parameters --- #

import scienceplots

plt.close("all")
plt.style.use(["science"])

# --- setup figure --- #

mm = 1 / 25.4
fig_width = 80
ratio = (1 + np.sqrt(5)) / 2
fig_height = fig_width / ratio

fig, ax = plt.subplots(
    2,
    1,
    figsize=(fig_width * mm, 2 * fig_height * mm),
    constrained_layout=True,
    sharex=True,
)
ax1, ax2 = ax

verbose = False

from lmfit import Minimizer, Parameters

T_arr = np.array([1.85, 1.95, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.75])


omega0 = np.array(
    [
        2.28317976e10,
        2.28303017e10,
        2.28270765e10,
        2.28204706e10,
        2.28097214e10,
        2.27936844e10,
        2.27710808e10,
        2.27387539e10,
        2.26981170e10,
        2.26465786e10,
        2.25645505e10,
    ]
)

kappa0 = np.array(
    [
        967601.66977029,
        1043306.54851011,
        1260122.10891764,
        1721468.34926133,
        2377237.36814291,
        3448387.82382859,
        5594472.88396356,
        7956546.09874414,
        7767876.86694651,
        9476161.97208435,
        14046586.1616838,
    ]
)

kappa_ext = np.array(
    [
        6214990.33440513,
        6139689.533781,
        6022935.61088472,
        5808098.52811428,
        5509924.09964161,
        5270987.07166838,
        4942570.520695,
        5219578.33923591,
        6401885.21866565,
        7466055.95630201,
        6525155.12712433,
    ]
)

# T_arr=np.take(T_arr,[0,1,2,3,4,5,8,9,10])
# omega0=np.take(omega0,[0,1,2,3,4,5,8,9,10])
# kappa0=np.take(kappa0,[0,1,2,3,4,5,8,9,10])
# kappa_ext=np.take(kappa_ext,[0,1,2,3,4,5,8,9,10])
data = np.concatenate((omega0, kappa0, kappa_ext))
# T_arr_fine=T_arr
T_arr = T_arr * 0.99773292 - 0.059
T_arr_fine = np.linspace(T_arr[0], T_arr[-1], 20)

pars = Parameters()

pars.add("Tc", value=9.9, min=0, vary=True)
pars.add("w00", value=omega0[0], vary=True)

"""fit resonance frequency ok """


def w0_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    w00 = params["w00"]
    D0 = a * Tc * KB
    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))
    if data is None:
        return w0
    else:
        return data - w0


def w0_T_TTF(params, T, data=None):
    # a=params['a']
    Tc = params["Tc"]
    w00 = params["w00"]
    # D0=a*Tc*KB
    w0 = w00 / np.sqrt(fit_T_dep.fL1(T, Tc))
    if data is None:
        return w0
    else:
        return data - w0


pars_w0 = pars
pars_w0.add("a", value=1.85, min=1, vary=False)
fitter = Minimizer(w0_T, pars_w0, fcn_args=(T_arr, omega0))
result = fitter.minimize()
fit_pars_w0 = result.params
w_fit = w0_T(fit_pars_w0, T_arr_fine)

fitter_TTF = Minimizer(w0_T_TTF, pars, fcn_args=(T_arr, omega0))
result_TTF = fitter_TTF.minimize()
fit_pars_w0_TTF = result_TTF.params
w_TTF = w0_T_TTF(fit_pars_w0_TTF, T_arr_fine)


# fit_pars=pars
Tc = fit_pars_w0["Tc"]
a = fit_pars_w0["a"]
w00 = fit_pars_w0["w00"]

if verbose:
    print(fit_pars_w0)

(line,) = ax1.plot(
    T_arr, omega0 / 2 / np.pi * 1e-9, "o", ms=4, alpha=1.0, label="Data", color="C0"
)
ax1.plot(T_arr_fine, w_fit / 2 / np.pi * 1e-9, "-", color="C1", label="MB")
ax1.plot(T_arr_fine, w_TTF / 2 / np.pi * 1e-9, "--", color="C2", label="TTF")

ax1.set_ylabel("$\omega_0 / 2 \pi$ (GHz)")
# ax1.annotate(
#     f"$T_c$ = {Tc.value:.2f} K (MB)",
#     xy=(0.45, 0.2),
#     xycoords="axes fraction",
#     fontsize=8,
# )
# ax1.annotate(
#     f"$T_c$ = {result_TTF.params['Tc'].value:.2f} K (TTF)",
#     xy=(0.45, 0.1),
#     xycoords="axes fraction",
#     fontsize=8,
# )
ax1.legend(frameon=False, loc="upper right", fontsize=8)

# --- insert schematic --- #

schem = plt.imread("schematic_circuit_parallel_lc_cc.png", format="png")
axin = ax1.inset_axes([0.02, 0.1, 0.4, 0.5], transform=ax1.transAxes)
axin.imshow(schem)
axin.axis("off")


# --- fit kappas --- #

k_data = np.concatenate((kappa_ext, kappa0))

Cc_guess = 13.8e-15
# C_guess=7.91e-13
# L0_guess=1/(w00**2*(C_guess+Cc_guess))
# L0_guess=2.38e-9
L0_guess = 2.38e-9
Rd_guess = 1738
pars_k = fit_pars_w0
pars_k["a"].vary = False
pars_k["Tc"].vary = False
pars_k["w00"].vary = False
pars_k.add("L0", value=L0_guess, min=0, vary=False)
# pars_k.add('C', value=C_guess, min=0, vary=True)
pars_k.add("Cc", value=Cc_guess, min=0, vary=False)
pars_k.add("Rd", value=Rd_guess, min=0, vary=False)
pars_k.add("beta", value=15.6, min=0, vary=False)
pars_k.add("Rd0", value=1.68e6, min=0, vary=False)


def k_T(params, T, data=None):
    Cc = params["Cc"]
    # C = params["C"]
    Rd = params["Rd"]
    Rd0 = params["Rd0"]
    beta = params["beta"]
    a = params["a"]
    Tc = params["Tc"]
    L0 = params["L0"]

    D0 = a * KB * Tc
    RL0 = D0 * L0 * np.pi / hbar

    L = L0 * fit_T_dep.fL(T, Tc, D0)
    RL = RL0 * fit_T_dep.fR(T, Tc, D0)
    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))

    # ZCc=1/(1j*w0*Cc+1/Rd)
    Yext = 1 / (50 + 1 / (1j * w0 * Cc))
    # Yext=1/(50+ZCc)
    Yext0 = 1 / (50 + 1 / (1j * w00 * Cc))
    # Yext0=1/(50+ZCc[0])
    C = 1 / (w00**2 * L0) - np.imag(Yext0) / w00

    kex = np.real(Yext) / (C + np.imag(Yext) / w0)
    k0 = (1 / RL + 1 / Rd0 + 1 / (Rd * np.exp(beta / T))) * 1 / (C + np.imag(Yext) / w0)
    # k0=(1/RL)*1/(C+np.imag(Yext)/w0)
    # k0=(1/(1/RL+1/Rd))/L#*1/(C+np.imag(Yext)/w0)

    if data is None:
        return np.concatenate((kex, k0))
    else:
        return np.concatenate((kex, k0)) - data


fitter_k = Minimizer(k_T, pars_k, fcn_args=(T_arr, k_data))
result_k = fitter_k.minimize()

fit_pars_k = result_k.params
# fit_pars_k=pars_k

kex_fit, k0_fit = np.split(k_T(fit_pars_k, T_arr_fine), 2)
(line1,) = ax2.plot(
    T_arr,
    kappa_ext / 2 / np.pi * 1e-6,
    "^",
    ms=4,
    alpha=1.0,
    color="C0",
    label="$\kappa_{{\mathrm{{ext}}}}$",
)
ax2.plot(
    T_arr_fine,
    kex_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{\mathrm{{ext}}}}$ fit",
)
(line0,) = ax2.plot(
    T_arr,
    kappa0 / 2 / np.pi * 1e-6,
    "v",
    ms=4,
    alpha=1.0,
    color="C0",
    label="$\kappa_{{0}}$",
)
ax2.plot(
    T_arr_fine,
    k0_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    label="fit",
)

if verbose:
    print(fit_pars_k)

fit_pars_k0_QP = fit_pars_k
fit_pars_k0_QP = result_k.params
fit_pars_k0_QP["Rd"].value = np.inf
fit_pars_k0_QP["Rd0"].value = np.inf

kex_fit2, k0_QP = np.split(k_T(fit_pars_k0_QP, T_arr_fine), 2)

ax2.plot(
    T_arr_fine,
    k0_QP / 2 / np.pi * 1e-6,
    "-",
    # label="MB pred.",
    color="C1",
)


# --- annotate curves --- #
ax2.annotate(
    "$\kappa_{{\mathrm{{ext}}}}$",
    # xy=(2.6, 1.5),
    # xytext=(1.8, 1.5),
    xy=(2, 1.5),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=0.0",
    # ),
)

ax2.annotate(
    "$\kappa_{{0}}$",
    # xy=(2.6, 0.3),
    # xytext=(1.8, 0.09),
    xy=(2, 0.1),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=0.1",
    # ),
)

ax2.annotate(
    "$\kappa_{{0}}^{{\mathrm{{QP}}}}$",
    # xy=(2.7, 0.10),
    # xytext=(1.8, 0.03),
    xy=(2.2, 1e-2),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=-0.01",
    # ),
)

# --- customise --- #

ax2.set_yscale("log")
ax2.set_xlabel("Temperature (K)")
ax2.set_ylabel("$\kappa / 2 \pi$ (MHz)")
ax2.set_ylim(0.5e-2, None)
ax2.legend(frameon=False, fontsize=8, loc="lower right")


# --- annotate sublabels --- #
annotate_params = dict(xy=(-0.2, 0.9), xycoords="axes fraction", annotation_clip=False)
ax1.annotate(r"\textbf{(a)}", **annotate_params)
ax2.annotate(r"\textbf{(b)}", **annotate_params)

# --- save --- #
fig.savefig("figure4_parallel_lc_cc.pdf", dpi=600)
